@@ -1,8 +1,8 @@
 //! Traits and functions for general purpose, everyday mathematics.
 //! Everything you need.
 
-use std::{convert::{TryInto, TryFrom}, ops::{Div, Sub, Add, Mul, AddAssign, SubAssign}};
-
+use std::{convert::{TryInto, TryFrom}, ops::*, cmp::*};
+use super::complex::ComplexNumber;
 use crate::eval_postfix;
 
 use super::constants::{SQRT5, GOLDENRATIO};
@@ -80,7 +80,6 @@ pub trait Averages<T> {
 /// Trait for the Digits of a given number.
 pub trait NumDigits {
     /// Calculates the cross sum of a number.
-    ///
     /// # Returns
     /// A `Self` containing the result.
     /// 
@@ -95,7 +94,6 @@ pub trait NumDigits {
     #[must_use]
     fn cross_sum(&self) -> Self;
     /// Gets the digits as a number with the 1s place at index 0.
-    ///
     /// # Returns
     /// A `Vec<u8>`.
     /// 
@@ -137,21 +135,17 @@ pub trait NumTools<T> {
     #[must_use]
     fn is_in_range_exclusive(&self, start: Self, end: Self) -> bool;
     /// Maps a given number of a range onto another range.
-    ///
     /// # Arguments
     /// * `self` - The value which is to be mapped.
     /// * `start1` - The original start value of the number range.
     /// * `end1` - The original end value of the number range.
     /// * `start2` - The new start value of the number range.
     /// * `end2` - The new start value of the number range.
-    ///
     /// # Returns
     /// A number containing the new mapped value.
-    ///
     /// # Examples
     /// ```
     /// use lib_rapid::math::general::NumTools;
-    ///
     /// let result: f32 = 5.0.map_to(0., 10., 0., 1.); // Original value 5 in the range from 0-10
     /// assert_eq!(result, 0.5);
     /// ```
@@ -219,7 +213,6 @@ pub trait NumTools<T> {
     ///use lib_rapid::math::general::NumTools;
     ///let i = 12;
     ///let res = i.square();
-    ///
     ///assert_eq!(144, res);
     /// ```
     #[must_use]
@@ -235,14 +228,13 @@ pub trait NumTools<T> {
     ///use lib_rapid::math::general::NumTools;
     ///let i = 12;
     ///let res = i.cube();
-    ///
     ///assert_eq!(1728, res);
     /// ```
     #[must_use]
     fn cube(&self) -> Self;
 }
 
-impl<T: std::cmp::PartialOrd +
+impl<T: PartialOrd +
         Sub<Output = T> +
         Add<Output = T> +
         Div<Output = T> +
@@ -253,12 +245,14 @@ impl<T: std::cmp::PartialOrd +
         AddAssign> NumTools<T> for T {
     #[inline]
     fn is_in_range(&self, start: Self, end: Self) -> bool {
-        self >= &start && self <= &end
+        self >= &start &&
+        self <= &end
     }
 
     #[inline]
     fn is_in_range_exclusive(&self, start: Self, end: Self) -> bool {
-        self > &start && self < &end
+        self > &start &&
+        self < &end
     }
 
     #[inline]
@@ -294,10 +288,10 @@ impl<T: std::cmp::PartialOrd +
     }
 }
 
-impl<'a, T: std::cmp::PartialEq +
-        std::ops::DivAssign + std::cmp::PartialOrd +
-        std::ops::Rem<Output = T> + Copy +
-        std::ops::AddAssign +
+impl<'a, T: PartialEq +
+        DivAssign + PartialOrd +
+        Rem<Output = T> + Copy +
+        AddAssign +
         Div<Output = T> +
         Sub<Output = T> +
         Add<Output = T> +
@@ -310,8 +304,8 @@ impl<'a, T: std::cmp::PartialEq +
         u8: TryFrom<T>,
         T: From<u8> {
     fn cross_sum(&self) -> Self {
-        let d: Vec<u8> = self.digits();
-        let mut res: Self = 0.into();
+        let     d:   Vec<u8> = self.digits();
+        let mut res: Self    = 0.into();
         for i in d {
             res.inc_by(i.into());
         }
@@ -387,4 +381,93 @@ pub fn nth_fibonacci(n: u128) -> u128 {
     }
 
     z
+}
+
+/// Compute the absolute difference ("Delta") between two values.
+/// # But Rust already has a abs_diff() function!
+/// Yes, it does. But it does not have support for floating point numbers.
+/// That's why we implemented a generic function.
+/// # Arguments
+/// - `a: T` - The first value.
+/// - `b: T` - The second value.
+/// # Returns
+/// A `T`.
+/// # Examples
+/// ```
+/// use lib_rapid::math::general::delta;
+/// 
+/// assert_eq!(4, delta(4, 8));
+/// assert_eq!(3.14, delta(6.28, 3.14));
+/// assert_eq!(4.905, delta(4.905, 9.81));
+/// ```
+#[must_use]
+#[inline]
+pub fn delta<T: Sub<Output = T> +
+                PartialOrd>(a: T, b: T) -> T {
+    match a > b {
+        true  => { return a - b; }
+        false => { return b - a; }
+    }
+}
+
+/// Panics if a given value `x` is odd.
+/// # Arguments
+/// - `x: T` - The number to be checked.
+/// # Examples
+/// ```should_panic
+/// use lib_rapid::math::general::better_be_even;
+/// 
+/// better_be_even(-3); // Panics, because -3 is odd.
+/// ```
+#[inline]
+pub fn better_be_even<T: From<bool> +
+                         BitAnd<Output = T> +
+                         PartialEq +
+                         std::fmt::Display +
+                         Copy>(x: T) {
+    if x & true.into() != false.into()
+    { core::panic!("Oops! {} was not even.", x); }
+}
+
+/// Allows for square roots of negative numbers.
+/// # Arguments
+/// - `x: f64` - The number of which the square root needs to be drawn.
+/// # Returns
+/// A `ComplexNumber<f64>`.
+/// # Examples
+/// ```
+/// use lib_rapid::math::{general::sqrt_f64, complex::ComplexNumber, constants::SQRT2};
+/// 
+/// let c = ComplexNumber::new(0.0f64, SQRT2);
+/// 
+/// assert_eq!(sqrt_f64(-2.0), c);
+/// ```
+#[inline]
+#[must_use]
+pub fn sqrt_f64(x: f64) -> ComplexNumber<f64> {
+    if x < 0.0
+    { return ComplexNumber { real: 0.0, complex: (-x).sqrt() }; }
+
+    ComplexNumber { real: x.sqrt(), complex: 0.0 }
+}
+/// Allows for square roots of negative numbers.
+/// # Arguments
+/// - `x: f64` - The number of which the square root needs to be drawn.
+/// # Returns
+/// A `ComplexNumber<f64>`.
+/// # Examples
+/// ```
+/// use lib_rapid::math::{general::sqrt_f32, complex::ComplexNumber, constants::SQRT2};
+/// 
+/// let c = ComplexNumber::new(0.0f32, SQRT2 as f32);
+/// 
+/// assert_eq!(sqrt_f32(-2.0), c);
+/// ```
+#[inline]
+#[must_use]
+pub fn sqrt_f32(x: f32) -> ComplexNumber<f32> {
+    if x < 0.0
+    { return ComplexNumber { real: 0.0, complex: (-x).sqrt() }; }
+
+    ComplexNumber { real: x.sqrt(), complex: 0.0 }
 }
\ No newline at end of file
